@@ -1,14 +1,17 @@
 # Set up __init__.py so that users can do from guardrails import Response, Schema, etc.
 
-from guardrails.guardrails import Guard
-from guardrails.utils import constants, docs_utils, rail_utils
+from guardrails.guard import Guard
+from guardrails.llm_providers import PromptCallable
+from guardrails.rail import Rail
+from guardrails.utils import constants, docs_utils
 from guardrails.validators import Validator, register_validator
 
 __all__ = [
     "Guard",
+    "PromptCallable",
+    "Rail",
     "Validator",
     "register_validator",
-    "rail_utils",
     "constants",
     "docs_utils",
 ]
