@@ -12,10 +12,10 @@ class PromptCallableException(Exception):
 
 @dataclass
 class PromptCallable:
-    """
-    A wrapper around a callable that takes in a prompt.
-    Catches exceptions to let the user know clearly
-    if the callable failed, and how to fix it.
+    """A wrapper around a callable that takes in a prompt.
+
+    Catches exceptions to let the user know clearly if the callable
+    failed, and how to fix it.
     """
 
     fn: Callable
@@ -26,7 +26,7 @@ class PromptCallable:
         except Exception as e:
             raise PromptCallableException(
                 "The callable `fn` passed to `Guard(fn, ...)` failed"
-                f" with the following error: {e}. "
+                f" with the following error: `{e}`. "
                 "Make sure that `fn` can be called as a function that"
                 " takes in a single prompt string "
                 "and returns a string."
@@ -70,9 +70,11 @@ def openai_chat_wrapper(text: str, *args, model="gpt-3.5-turbo", **kwargs):
 
 def get_llm_ask(llm_api: Callable, *args, **kwargs):
     if llm_api == openai.Completion.create:
-        return partial(openai_wrapper, *args, **kwargs)
+        fn = partial(openai_wrapper, *args, **kwargs)
     elif llm_api == openai.ChatCompletion.create:
-        return partial(openai_chat_wrapper, *args, **kwargs)
+        fn = partial(openai_chat_wrapper, *args, **kwargs)
     else:
         # Let the user pass in an arbitrary callable.
-        return PromptCallable(fn=partial(llm_api, *args, **kwargs))
+        fn = partial(llm_api, *args, **kwargs)
+
+    return PromptCallable(fn=fn)
