@@ -10,13 +10,13 @@ class GuardLogs:
     prompt: str
     output: str
     output_as_dict: dict
-    validated_response: dict
+    validated_output: dict
     reasks: List[ReAsk]
 
     @property
     def failed_validations(self) -> List[ReAsk]:
         """Returns the failed validations."""
-        return gather_reasks(self.validated_response)
+        return gather_reasks(self.validated_output)
 
 
 @dataclass
@@ -26,14 +26,14 @@ class GuardHistory:
     def push(self, guard_log: GuardLogs) -> "GuardHistory":
         if len(self.history) > 0:
             last_log = self.history[-1]
-            guard_log.validated_response = merge_reask_output(last_log, guard_log)
+            guard_log.validated_output = merge_reask_output(last_log, guard_log)
 
         return GuardHistory(self.history + [guard_log])
 
     @property
-    def validated_response(self) -> dict:
+    def validated_output(self) -> dict:
         """Returns the latest validated output."""
-        return self.history[-1].validated_response
+        return self.history[-1].validated_output
 
     @property
     def output(self) -> str:
@@ -89,9 +89,11 @@ def merge_reask_output(prev_logs: GuardLogs, current_logs: GuardLogs) -> Dict:
     Returns:
         The merged output.
     """
-    previous_response = prev_logs.validated_response
+    from guardrails.validators import PydanticReAsk
+
+    previous_response = prev_logs.validated_output
     pruned_reask_json = prune_json_for_reasking(previous_response)
-    reask_response = current_logs.validated_response
+    reask_response = current_logs.validated_output
 
     # Reask output and reask json have the same structure, except that values
     # of the reask json are ReAsk objects. We want to replace the ReAsk objects
@@ -99,7 +101,16 @@ def merge_reask_output(prev_logs: GuardLogs, current_logs: GuardLogs) -> Dict:
     merged_json = deepcopy(previous_response)
 
     def update_reasked_elements(pruned_reask_json, reask_response_dict):
-        if isinstance(pruned_reask_json, dict):
+        if isinstance(pruned_reask_json, PydanticReAsk):
+            corrected_value = reask_response_dict
+            # Get the path from any of the ReAsk objects in the PydanticReAsk object
+            # all of them have the same path.
+            path = [v.path for v in pruned_reask_json.values() if isinstance(v, ReAsk)][
+                0
+            ]
+            update_response_by_path(merged_json, path, corrected_value)
+
+        elif isinstance(pruned_reask_json, dict):
             for key, value in pruned_reask_json.items():
                 if isinstance(value, ReAsk):
                     corrected_value = reask_response_dict[key]
