@@ -7,12 +7,24 @@ import ast
 import logging
 import os
 from collections import defaultdict
+from copy import deepcopy
 from dataclasses import dataclass
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Any, Callable, Dict, List, Optional, Type, Union
+
+import openai
+from pydantic import BaseModel, ValidationError
 
 from guardrails.datatypes import registry as types_registry
 from guardrails.utils.reask_utils import ReAsk
 
+try:
+    import numpy as np
+except ImportError:
+    _HAS_NUMPY = False
+else:
+    _HAS_NUMPY = True
+
+
 validators_registry = {}
 types_to_validators = defaultdict(list)
 
@@ -88,6 +100,8 @@ def filter_in_list(schema: List) -> List:
     for item in schema:
         if isinstance(item, Filter):
             pass
+        elif isinstance(item, PydanticReAsk):
+            filtered_list.append(item)
         elif isinstance(item, list):
             filtered_item = filter_in_list(item)
             if len(filtered_item):
@@ -117,6 +131,8 @@ def filter_in_dict(schema: Dict) -> Dict:
     for key, value in schema.items():
         if isinstance(value, Filter):
             pass
+        elif isinstance(value, PydanticReAsk):
+            filtered_dict[key] = value
         elif isinstance(value, list):
             filtered_item = filter_in_list(value)
             if len(filtered_item):
@@ -148,6 +164,7 @@ def register_validator(name: str, data_type: Union[str, List[str]]):
             types_to_validators[dt].append(name)
 
         validators_registry[name] = cls
+        cls.rail_alias = name
         return cls
 
     return decorator
@@ -167,24 +184,18 @@ class EventDetail(BaseException):
 class Validator:
     """Base class for validators."""
 
-    def __init__(self, on_fail: Optional[Callable] = None):
-        if on_fail is not None:
-            if isinstance(on_fail, str):
-                if on_fail == "filter":
-                    on_fail = self.filter
-                elif on_fail == "refrain":
-                    on_fail = self.refrain
-                elif on_fail == "noop":
-                    on_fail = self.noop
-                elif on_fail == "fix":
-                    on_fail = self.fix
-                elif on_fail == "reask":
-                    on_fail = self.reask
-                else:
-                    raise ValueError(f"Unknown on_fail value: {on_fail}.")
-            self.on_fail = on_fail
+    def __init__(self, on_fail: Optional[Callable] = None, **kwargs):
+        if isinstance(on_fail, str):
+            self.on_fail = getattr(self, on_fail, self.noop)
         else:
-            self.on_fail = self.fix
+            self.on_fail = on_fail or self.noop
+
+        # Store the kwargs for the validator.
+        self._kwargs = kwargs
+
+        assert (
+            self.rail_alias in validators_registry
+        ), f"Validator {self.__class__.__name__} is not registered. "
 
     def validate_with_correction(self, key, value, schema) -> Dict:
         try:
@@ -244,6 +255,13 @@ class Validator:
 
         return error.schema
 
+    def to_prompt(self, with_keywords: bool = True) -> str:
+        """Convert the validator to a prompt."""
+        params = " ".join(list(self._kwargs.values()))
+        if with_keywords:
+            params = " ".join([f"{k}={v}" for k, v in self._kwargs.items()])
+        return f"{self.rail_alias}: {params}"
+
 
 # @register_validator('required', 'all')
 # class Required(Validator):
@@ -265,6 +283,90 @@ class Validator:
 #         return value is not None
 
 
+class PydanticReAsk(dict):
+    pass
+
+
+@register_validator(name="pydantic", data_type="pydantic")
+class Pydantic(Validator):
+    """Validate an object using Pydantic."""
+
+    def __init__(
+        self,
+        model: Type[BaseModel],
+        on_fail: Optional[Callable] = None,
+    ):
+        super().__init__(on_fail=on_fail)
+
+        self.model = model
+
+    def validate_with_correction(
+        self, key: str, value: Dict, schema: Union[Dict, List]
+    ) -> Dict:
+        """Validate an object using Pydantic.
+
+        For example, consider the following data for a `Person` model
+        with fields `name`, `age`, and `zipcode`:
+        {
+            "user" : {
+                "name": "John",
+                "age": 30,
+                "zipcode": "12345",
+            }
+        }
+        then `key` is "user", `value` is the value of the "user" key, and
+        `schema` is the entire schema.
+
+        If this validator succeeds, then the `schema` is returned and
+        looks like:
+        {
+            "user": Person(name="John", age=30, zipcode="12345")
+        }
+
+        If it fails, then the `schema` is returned and looks like e.g.
+        {
+            "user": {
+                "name": "John",
+                "age": 30,
+                "zipcode": ReAsk(
+                    incorrect_value="12345",
+                    error_message="...",
+                    fix_value=None,
+                    path=None,
+                )
+            }
+        }
+        """
+        try:
+            # Run the Pydantic model on the value.
+            schema[key] = self.model(**value)
+        except ValidationError as e:
+            # Create a copy of the value so that we can modify it
+            # to insert e.g. ReAsk objects.
+            new_value = deepcopy(value)
+            for error in e.errors():
+                assert (
+                    len(error["loc"]) == 1
+                ), "Pydantic validation errors should only have one location."
+
+                field_name = error["loc"][0]
+                event_detail = EventDetail(
+                    key=field_name,
+                    value=new_value[field_name],
+                    schema=new_value,
+                    error_message=error["msg"],
+                    fix_value=None,
+                )
+                # Call the on_fail method and reassign the value.
+                new_value = self.on_fail(event_detail)
+
+            # Insert the new `value` dictionary into the schema.
+            # This now contains e.g. ReAsk objects.
+            schema[key] = PydanticReAsk(new_value)
+
+        return schema
+
+
 @register_validator(name="valid-range", data_type=["integer", "float", "percentage"])
 class ValidRange(Validator):
     """Validate that a value is within a range.
@@ -588,6 +690,82 @@ class BugFreeSQL(Validator):
         return schema
 
 
+@register_validator(name="similar-to-document", data_type="string")
+class SimilarToDocument(Validator):
+    """Validate that a value is similar to the document.
+
+    This validator checks if the value is similar to the document by checking
+    the cosine similarity between the value and the document, using an
+    embedding.
+
+    - Name for `format` attribute: `similar-to-document`
+    - Supported data types: `string`
+    - Programmatic fix: None
+    """
+
+    def __init__(
+        self,
+        document: str,
+        threshold: float = 0.7,
+        model: str = "text-embedding-ada-002",
+        on_fail: Optional[Callable] = None,
+    ):
+        super().__init__(on_fail=on_fail)
+        if not _HAS_NUMPY:
+            raise ImportError(
+                f"The {self.__class__.__name__} validator requires the numpy package.\n"
+                "`pip install numpy` to install it."
+            )
+
+        self._document = document
+        embedding = openai.Embedding.create(input=[document], model=model)["data"][0][
+            "embedding"
+        ]
+        self._document_embedding = np.array(embedding)
+        self._model = model
+        self._threshold = float(threshold)
+
+    @staticmethod
+    def cosine_similarity(a: "np.ndarray", b: "np.ndarray") -> float:
+        """Calculate the cosine similarity between two vectors.
+
+        Args:
+            a: The first vector.
+            b: The second vector.
+
+        Returns:
+            float: The cosine similarity between the two vectors.
+        """
+        return np.dot(a, b) / (np.linalg.norm(a) * np.linalg.norm(b))
+
+    def validate(self, key: str, value: Any, schema: Union[Dict, List]) -> Dict:
+        logger.debug(f"Validating {value} is similar to document...")
+
+        value_embedding = np.array(
+            openai.Embedding.create(input=[value], model=self._model)["data"][0][
+                "embedding"
+            ]
+        )
+
+        similarity = SimilarToDocument.cosine_similarity(
+            self._document_embedding,
+            value_embedding,
+        )
+        if similarity < self._threshold:
+            raise EventDetail(
+                key,
+                value,
+                schema,
+                f"Value {value} is not similar enough to document {self._document}.",
+                None,
+            )
+
+        return schema
+
+    def to_prompt(self, with_keywords: bool = True) -> str:
+        return ""
+
+
 @register_validator(name="is-profanity-free", data_type="string")
 class IsProfanityFree(Validator):
     """Validate that a translated text does not contain profanity language.
@@ -660,3 +838,31 @@ class IsHighQualityTranslation(Validator):
                 "",
             )
         return schema
+
+
+@register_validator(name="ends-with", data_type="list")
+class EndsWith(Validator):
+    """Validate that a list ends with a given value.
+
+    - Name for `format` attribute: `ends-with`
+    - Supported data types: `list`
+    - Programmatic fix: Append the given value to the list.
+    """
+
+    def __init__(self, end: str, on_fail: str = "fix"):
+        super().__init__(on_fail=on_fail)
+        self._end = end
+
+    def validate(self, key: str, value: Any, schema: Union[Dict, List]) -> Dict:
+        logger.debug(f"Validating {value} ends with {self._end}...")
+
+        if not value[-1] == self._end:
+            raise EventDetail(
+                key,
+                value,
+                schema,
+                f"{value} must end with {self._end}",
+                value + [self._end],
+            )
+
+        return schema
